;;; BLRN -> Block rename (by object selection)
;;; BX -> Change "Allow exploding" state (by object selection)
(vl-load-com)

(defun C:BLRN (/ OLDERR OBJ DT OLD)
  (setq OLDERR *error* *error* BLRN_Error)
  (if (setq OBJ (entsel "\nSelect block to rename: "))
    (progn
      (setq DT (entget (car OBJ)))
      (if (= (cdr (assoc 0 DT)) "INSERT")
        (progn
          (setq OLD (cdr (assoc 2 DT)))
          (if (= (jk:BLK_XRefStat OLD) nil)
            (progn
              (if (> (atof (substr (getvar "ACADVER") 1 4)) 16.1)
                (setq OLD
                  (vlax-get-property
                    (vlax-ename->vla-Object (car OBJ))
                    "effectivename"
                  )
                )
                (setq OLD (cdr (assoc 2 DT)))
              )
              (if 
                (or
                  (/= (substr OLD 1 2) "*U")
                  (>= (atof (substr (getvar "ACADVER") 1 4)) 17.2)
                )
                 (if
                   (vlax-write-enabled-p
                     (car OBJ)
                   )
                   (progn
                     (jk:SYS_UndoBegin)
                     (vla-highlight
                       (vlax-ename->vla-object (car OBJ))
                       :vlax-true
                     )
                     (redraw)
                     (kr:BLRN_Dialog)
                     (jk:SYS_UndoEnd)
                   )
                   (princ "\n>> Object on a locked layer. ")
                 )
                 (princ "\n>> Can't rename anonymous block. ")
              )
            )
            (princ (strcat "\n>> Caution! " (strcase OLD) " is an externally referenced block. "))
          )
        )
        (princ "\n>> Selected item not an INSERT. ")
      )
    )
    (princ "\n>> Nothing selected. ")
  )
  (if OLDERR (setq *error* OLDERR))
  (princ)
)

; ============================================================ ;
; Obsuga okna dialogowego programu BLRN                       ;
; ============================================================ ;
(defun kr:BLRN_Dialog (/ DCL BL_LIST NEW TEMP TEMP# EXPL ACT)
  (if (not ai_table)
    (load "ai_utils.lsp" T)
  )
  (setq DCL (load_dialog "Block Rename.dcl")
        BL_LIST (acad_strlsort (ai_table "BLOCK" (+ 2 4 8)))
        TEMP OLD
  )
  (if
    (not (new_dialog "BLRN" DCL))
    (alert "Can't find Block Rename.dcl")
    (progn
      (set_tile "BLRN" OLD)
      (set_tile "OLD" OLD)
      (set_tile "error" "Block name already exist.")
      (mode_tile "BLRN" 2)
      ;(mode_tile "OK" 1)
      (if BL_LIST
        (mode_tile "BL_LIST" 0)
        (mode_tile "BL_LIST" 1)
      )
      (jk:DLG_PutList "BL_LIST" BL_LIST (vl-position OLD BL_LIST))
      (action_tile "BLRN" "(setq NEW (kr:BLRN_CheckName $value))")
      (action_tile "BL_LIST" "(setq TEMP (nth (atoi $value) BL_LIST))
                              (set_tile \"BLRN\" TEMP)
                              (set_tile \"error\" \"Block name already exist.\")
                              ;(mode_tile \"OK\" 1)
                              "
      )
      (action_tile "OK" "(if NEW (done_dialog 1))")
      (action_tile "SELECT" "(done_dialog 2)")
      (action_tile "CLOSE" "(done_dialog 0)")
      (setq ACT (start_dialog))
      (unload_dialog DCL)
      (vla-highlight
        (vlax-ename->vla-object (car OBJ))
        :vlax-false
      )
      (cond
        ( (zerop ACT)
          (princ "\n>> No items renamed. ")
        )
        ( (= ACT 1)
          (vla-put-name
            (vlax-ename->vla-object
              (cdr
                (assoc 330 (entget (tblObjname "BLOCK" OLD)))
              )
            )
            NEW
          )
          (princ (strcat "\n>> Block <" (strcase OLD) "> renamed to <" NEW ">."))
        )
        ( (= ACT 2)
          (princ "\n>> No items renamed. ")
          (C:BLRN)
        )
      )
    )
  )
)

; ============================================================ ;
; Sprawdza poprawno nowej nazwy bloku                        ;
; ============================================================ ;
(defun kr:BLRN_CheckName (Val / TST RES)
  (setq TST Val)
  (if (= (snvalid Val) nil)
    (progn
      (set_tile "error" "Invalid new block name.")
      ;(mode_tile "OK" 1)
      (setq RES nil)
    )
    (if (tblsearch "BLOCK" Val)
      (progn
        (set_tile "error" "Block name already exist.")
        ;(mode_tile "OK" 1)
        (setq RES nil)
      )
      (progn
        (set_tile "error" "")
        ;(mode_tile "OK" 0)
        (setq RES TST)
      )
    )
  )
  RES
)

; ============================================================ ;
; Sprawdza czy blok moe/niemoe by rozbity (Allow exploding) ;
; ============================================================ ;
(defun kr:BLRN_CheckExp (Name / BLOCKS BLK AE)
  (setq BLOCKS (vla-get-Blocks (jk:ACX_ActDoc)))
  (setq BLK (vla-Item BLOCKS Name))
  (if (= (vla-get-explodable BLK) :vlax-true)
    (setq AE "YES")
    (setq AE "NO")
  )
  AE
)

; ============================================================ ;
; Obsuga bedw BLRN                                          ;
; ============================================================ ;
(defun BLRN_Error (msg)
  (if
    (= 8 (logand 8 (getvar "UNDOCTL")))
    (jk:SYS_UndoEnd)
  )
  (if
    (or (= msg "Function cancelled")(= msg "quit / exit abort"))
    (princ (strcat "\n>> Error: " msg))
    (princ "\n>> Cancelled. ")
  )
  (if OLDERR (setq *error* OLDERR))
  (if OBJ
    (vla-highlight
      (vlax-ename->vla-object (car OBJ))
      :vlax-false
    )
  )
  (princ)
)

; ============================================================ ;
; Obsuga C:BX                                                 ;
; ============================================================ ;
(defun C:BX (/ OLDERR OBJ DT IN IN#)
  (setq OLDERR *error* *error* BX_Error)
  (if (> (distof (substr (getvar "ACADVER") 1 4)) 16.1)
    (if (setq OBJ (entsel "\nSelect block to change 'Allow exploding' state: "))
      (progn
        (setq DT (entget (car OBJ)))
        (if (= (cdr (assoc 0 DT)) "INSERT")
          (progn
            (setq IN (cdr (assoc 2 DT)))
            (if (not (jk:BLK_XRefStat IN))
              (progn
                (setq IN
                  (vlax-get-property
                    (vlax-ename->vla-Object (car OBJ))
                    "effectivename"
                  )
                )
                (if
                  (vlax-write-enabled-p
                    (car OBJ)
                  )
                  (progn
                    (jk:SYS_UndoBegin)
                    (kr:BX_ChangeExp
                      (if
                        (setq IN# (jk:BLK_GetDynBlockNames IN))
                          IN#
                          IN
                      )
                      (kr:BLRN_CheckExp IN)
                    )
                    (jk:SYS_UndoEnd)
                  )
                  (princ "\n>> Object on a locked layer. ")
                )
              )
              (princ (strcat "\n>> Caution! " (strcase IN) " is an externally referenced block. "))
            )
          )
          (princ "\n>> Selected item not an INSERT. ")
        )
      )
      (princ "\n>> Nothing selected. ")
    )
    (princ "\n>> BX not available in this version. ")
  )
  (if OLDERR (setq *error* OLDERR))
  (princ)
)

; ============================================================ ;
; Blokuje/odblok. moliwo rozbicia bloku (Allow exploding)   ;
; ============================================================ ;
(defun kr:BX_ChangeExp (Name Val / BLOCKS BLK)
  (if (not (listp Name))
    (setq Name (list Name))
  )
  (setq BLOCKS (vla-get-Blocks (jk:ACX_ActDoc))
        BLK (vla-Item BLOCKS (car Name))
  )
  (cond
    ( (= Val "YES")
      (mapcar '(lambda (%) (vla-put-explodable (vla-Item BLOCKS %) :vlax-false)) Name)
      (princ "\n>> Block 'Allow exploding' changed to NO.")
    )
    ( (= Val "NO")
      (mapcar '(lambda (%) (vla-put-explodable (vla-Item BLOCKS %) :vlax-true)) Name)
        (princ "\n>> Block 'Allow exploding' changed to YES.")
    )
    (T nil)
  )
)

; ============================================================ ;
; Obsuga bedw BX                                            ;
; ============================================================ ;
(defun BX_Error (msg)
  (if
    (= 8 (logand 8 (getvar "UNDOCTL")))
    (jk:SYS_UndoEnd)
  )
  (if
    (or (= msg "Function cancelled")(= msg "quit / exit abort"))
    (princ (strcat "\n>> Error: " msg))
    (princ "\n>> Cancelled. ")
  )
  (if OLDERR (setq *error* OLDERR))
  (princ)
)

; ************************************************************ ;
; FUNKCJE BIBLIOTECZNE                                         ;
; ************************************************************ ;

; ============================================================ ;
; Funkcja dla argumentu bedacego EffectiveName bloku dyna-     ;
; micznego zwaraca liste wszystkich blokow zaleznych (typu *U) ;
; od tej definicji.                                            ;
; by kojacek (08-2010)                                         ;
; ============================================================ ;
(defun jk:BLK_GetDynBlockNames (Name / l %1 n)
  (setq l (list Name))
  (vlax-for %
    (vla-get-Blocks (jk:ACX_ActDoc))
    (if
      (wcmatch (setq n (vla-get-Name %)) "`*U*")
      (if
        (setq %1
          (jk:XDT_GetxData
            (vlax-vla-object->ename %)
            "AcDbBlockRepBTag"
          )
        )
        (if
          (= (strcase Name)
            (strcase
              (cdr
                (assoc 2
                  (entget
                    (handent
                      (cdr (assoc 1005 (cdr %1)))
                    )
                  )
                )
              )
            )
          )
          (setq l (cons n l))
        )
      )
    )
  )
  (if (> (length l) 1) (reverse l) nil)
)

; ============================================================ ;
; Czyta dane dodatkowe                                         ;
; ============================================================ ;
(defun jk:XDT_GetxData (en appName)
  (cadr
    (assoc -3 (entget en (list appName)))
  )
)

; ============================================================ ;
; Zwraca status XREF-a                                         ;
; ============================================================ ;
(defun jk:BLK_XRefStat (Name / Data Obj Res)
  (if
    (setq Obj (tblobjname "BLOCK" Name))
    (progn
      (setq Data (entget Obj))
      (if
        (= (logand (setq Res (cdr (assoc 70 Data))) 4) 4)
        Res
        Nil
      )
    )
    Nil 
  )
)

; ============================================================ ;
; Zwraca obiekt ActiveDocument aktualnego rysunku              ;
; ============================================================ ;
(defun jk:ACX_ActDoc ()
  (if
    (not *jk-ActDoc)
    (setq *jk-ActDoc
      (vla-get-activedocument
        (vlax-get-acad-object)
      )
    )
    *jk-ActDoc
  )
)

; ============================================================ ;
; UndoStart                                                    ;
; ============================================================ ;
(defun jk:SYS_UndoBegin ()
  (vla-StartUndoMark (jk:ACX_ActDoc))
)

; ============================================================ ;
; UndoEnd                                                      ;
; ============================================================ ;
(defun jk:SYS_UndoEnd ()
  (vla-EndUndoMark (jk:ACX_ActDoc))
)

; ============================================================ ;
; Procedura wypenia wycinek typu "list_box" i "popup_list"    ;
; Argumenty:                                                   ;
;          ListKey   - nazwa wycinka (Str)                     ;
;          FillList  - lista do wywietlenia (List)            ;
;          APosition - pozycja "aktualna"                      ;
;                      (Str reprezent. Int) lub Nil            ;
; ============================================================ ;
(defun jk:DLG_PutList (ListKey FillList Pos)
  (start_list ListKey)
  (mapcar 'add_list FillList)
  (end_list) 
  (if Pos
    (cond
      ( (= (type Pos) 'STR)
        (set_tile ListKey (itoa (atoi Pos)))
      )
      ( (= (type Pos) 'INT)
        (set_tile ListKey (itoa Pos))
      )
      (T Nil)
    ) 
  )
)

(princ)